//PercoLooper.cpp -- this file is the code for the Percolator Internal Loopers.

#include "Verbosity.h"
#include <String.h>
#include <Message.h>
#include <Messenger.h>
#include <MessageRunner.h>
#include <Looper.h>
#include <Application.h>
#include "PercoLooper.h"
#include "PercolatorCodes.h"
#include "PercoInternalMessaging.h"

BPercoLooper::BPercoLooper(BMessage *msg_generate)
{
  flg_Settings = B_PERCO_DEFAULT;
  
  BString *str_MimeType = new BString;
  BString *str_Name = new BString;
  msg_generate->FindString("Be:PERC:name",str_Name);	//extract name and mimetype.
  msg_generate->FindString("Be:PERC:mimetype",str_MimeType);

  lnk_Manager = new BMessenger();
  msg_generate->FindMessenger("Be:PERC:manager",lnk_Manager);	//Link to the manager.
  
  msg_Settings = new BMessage('_BST');			//setup the settings variables.
    msg_Settings->AddFloat("Be:PERC:width", 0);
    msg_Settings->AddFloat("Be:PERC:height", 0);
    msg_Settings->AddString("Be:PERC:name", *str_Name);
    msg_Settings->AddString("Be:PERC:mimetype", *str_MimeType);

  str_Name->Prepend("p:");  
  SetName(str_Name->String());
  cout << Name() << " created with mimetype :" << str_MimeType->String() << endl;
  
  lnk_Client = new BMessenger();
  msg_generate->FindMessenger("Be:PERC:linkback",lnk_Client);	//link to the client.
  
  lnk_This = new BMessenger(NULL, this);
  
  msr_Pinger = new BMessageRunner(*lnk_This, new BMessage(B_PERCOLATOR_PING), 100000);
  
  BMessage *msg_Redirect = new BMessage(B_PERCOLATOR_REDIRECT);
    msg_Redirect->AddMessenger("Be:PERC:redirect",*lnk_This);
    lnk_Client->SendMessage(msg_Redirect, this);
  delete msg_Redirect;

  
  TypechangeNotify(msg_generate);
    
  pingcount = 5;
};

BPercoLooper::~BPercoLooper()
{
  if (msg_Settings)
    delete msg_Settings;
  if (msr_Pinger)
    delete msr_Pinger;
  if (lnk_This)
    delete lnk_This;
  if (lnk_Manager)
    delete lnk_Manager;
  if (lnk_Client)
    delete lnk_Client;
};

void BPercoLooper::MessageReceived(BMessage *msg_received)
{
  switch(msg_received->what)
  {
    case B_PERCOLATOR_PING:
      PingReceived(msg_received);
    break;
    case B_PERCOLATOR_WAIT:
      WaitDirective();
    break;
    case B_PERCOLATOR_PING_REPLY:
      PingReplied(msg_received);
    break;
    case B_PERCOLATOR_TERMINATE_REQUEST:
      TerminateRequest(msg_received);
    break;
    case B_PERCOLATOR_RESIZE_NOTIFY:
      ResizeNotify(msg_received);
    break;
    case B_PERCOLATOR_TYPECHANGE_NOTIFY:
      TypechangeNotify(msg_received);
    break;
    case B_PERCOLATOR_IMAGE_REPLY:
      ImagechangeNotify(msg_received);
    break;
    default:
      BLooper::MessageReceived(msg_received);
  };
};

status_t BPercoLooper::Render()
{
  cout << "Rendering." << endl;
  flg_Status &= !(B_PERCO_NEEDS_REFRESH);
  return B_OK;
};

void BPercoLooper::WaitDirective()
{
  BMessage *msg_Delayed = new BMessage(B_PERCOLATOR_WAIT);
    lnk_Client->SendMessage(msg_Delayed, *lnk_This);		//forward the delayed notice
  delete msg_Delayed;									//the client.
  
  flg_Status |= B_PERCO_NEEDS_REFRESH;					//flag it for rerendering.
};

void BPercoLooper::PingReplied(BMessage *msg_pingreplied)
{
  pingcount = 5;
};

void BPercoLooper::PingReceived(BMessage *msg_ping)
{
  if (flg_Status & B_PERCO_NEEDS_REFRESH)
    Render();
  if (pingcount-- > 0)  //decrement the ping count.
    lnk_Client->SendMessage(msg_ping, *lnk_This);  //forward the ping
  else
  {
    cout << "Ping count expired.  Shutting down " << Name() << endl;
    BMessage *msg_SelfDestruct = new BMessage(B_PERCOLATOR_TERMINATE_REQUEST);
      msg_SelfDestruct->AddPointer("Be:PERC:destroy",this);
      be_app_messenger.SendMessage(msg_SelfDestruct);
    delete msg_SelfDestruct;
  };
};

void BPercoLooper::TerminateRequest(BMessage *msg_terminate)
{
  cout << "Normal shutdown of " << Name() << endl;
  msg_terminate->AddPointer("Be:PERC:destroy",this);
  be_app_messenger.SendMessage(msg_terminate);
};

void BPercoLooper::ResizeNotify(BMessage *msg_resize)
{
  float width, height;
  msg_Settings->FindFloat("Be:PERC:width", &width);
  msg_Settings->FindFloat("Be:PERC:height", &height);
  cout << "Resize: (" << width << "," << height << ") ->";
  msg_resize->FindFloat("Be:PERC:width", &width);
  msg_resize->FindFloat("Be:PERC:height", &height);
  cout << " (" << width << "," << height << ")" << endl;
  msg_Settings->ReplaceFloat("Be:PERC:width", width);
  msg_Settings->ReplaceFloat("Be:PERC:height", height);
  flg_Status |= B_PERCO_NEEDS_REFRESH;
};

void BPercoLooper::TypechangeNotify(BMessage *msg_typechange)
{
  BString str_MimeType;
  msg_typechange->FindString("Be:PERC:mimetype",&str_MimeType);
  
  cout << str_MimeType.String() << endl;
  
  msg_Settings->ReplaceString("Be:PERC:mimetype",str_MimeType.String());
    
  BMessage *msg_ImageRequest = new BMessage(B_PERCOLATOR_IMAGE_REQUEST);
    msg_ImageRequest->AddString("Be:PERC:mimetype",str_MimeType.String());
    VerboseReply(lnk_Manager->SendMessage(msg_ImageRequest,*lnk_This)
      ,"requesting typechange", true);
  delete msg_ImageRequest;
};

void BPercoLooper::ImagechangeNotify(BMessage *msg_imagechange)
{
  cout << "Image changed" << endl;
};
