#include <Errors.h>
#include <MediaDefs.h>
#include "Verbosity.h"

void VerboseReply(status_t status, char *milepost, bool full = false, char *clearmessage = NULL, void (*MoreErrors)(status_t status) = NULL)
{
  if ((status == B_OK) && (clearmessage) && (full))
    cout << clearmessage << endl;
  else
  {
    if ((full) || (status != B_OK))
    cout << milepost << "--";
    switch (status)
    {
      case B_OK:								if (full)
      											cout << "Successful" <<	endl;						break;
      case B_NO_MEMORY:							cout << "Out of memory" << endl;					break;
      case B_IO_ERROR:							cout << "General I/O error" << endl;				break;
      case B_PERMISSION_DENIED:					cout << "Illegal access" << endl;					break;
      case B_BAD_INDEX:							cout << "Index out of range" << endl;				break;
      case B_BAD_VALUE:							cout << "Illegal value" << endl;					break;
      case B_MISMATCHED_VALUES:					cout << "Conflicting values" << endl;				break;
      case B_BAD_TYPE:							cout << "Illegal argument type" << endl;			break;
      case B_NAME_NOT_FOUND:					cout << "Matching name not found" << endl;			break;
      case B_NAME_IN_USE:						cout << "Requested name used" << endl;				break;
      case B_BUSY:								cout << "Busy device" << endl;						break;
      case B_NOT_ALLOWED:						cout << "Operation not allowed" << endl;			break;
      case B_NO_INIT:							cout << "Improperly initialized object" << endl;	break;
      case B_TIMED_OUT:							cout << "Time expired before completion"<< endl;	break;
      case B_INTERRUPTED:						cout << "Signal interruption" << endl;				break;
      case B_WOULD_BLOCK:						cout << "Block intent error" << endl;				break;
      case B_ERROR:								cout << "General error" << endl;					break;
      
      case B_FILE_ERROR:						cout << "General file error" << endl;				break;
      case B_FILE_EXISTS:						cout << "File overwrite guard caught" << endl;		break;
      case B_ENTRY_NOT_FOUND:					cout << "File entry does not exist" << endl;		break;
      case B_NAME_TOO_LONG:						cout << "Leaf or pathname too long" << endl;		break;
      case B_NO_MORE_FDS:						cout << "No more file descriptors" << endl;			break;
      case B_NOT_A_DIRECTORY:					cout << "Path name is not a directory" << endl; 	break;
      case B_IS_A_DIRECTORY:					cout << "Path name is a directory" << endl;			break;
      case B_DIRECTORY_NOT_EMPTY:				cout << "Directory is not empty" << endl;			break;
      case B_DEVICE_FULL:						cout << "Device is full" << endl;					break;
      case B_READ_ONLY_DEVICE:					cout << "Read-only device" << endl;					break;
      case B_CROSS_DEVICE_LINK:					cout << "Cross link not allowed" << endl;			break;
      case B_LINK_LIMIT:						cout << "Nested links too deep" << endl;			break;
      case B_BUSTED_PIPE:						cout << "Busted pipe!" << endl;						break;
      case B_UNSUPPORTED:						cout << "Unsupported operation" << endl;			break;
      case B_PARTITION_TOO_SMALL:				cout << "Partition too small" << endl;				break;
      
      case B_DUPLICATE_REPLY:					cout << "Reply already sent" << endl;				break;
      case B_BAD_REPLY:							cout << "Inappropriate reply" << endl;				break;
      case B_BAD_HANDLER:						cout << "Handler does not exist" << endl;			break;
      case B_MESSAGE_TO_SELF:					cout << "Thread sending self message" << endl;		break;
      case B_ALREADY_RUNNING:					cout << "Application is single-launch" << endl; 	break;
      case B_LAUNCH_FAILED:						cout << "Attempt to launch app failed" << endl; 	break;
      case B_AMBIGUOUS_APP_LAUNCH:				cout << "Who ya gonna call? Appbusters!" << endl;	break;
      case B_UNKNOWN_MIME_TYPE:					cout << "Application signature unknown" << endl;	break;
      case B_BAD_SCRIPT_SYNTAX:					cout << "Script syntax malformed" << endl;			break;
      case B_LAUNCH_FAILED_NO_RESOLVE_LINK:		cout << "Cannot resolve app link" << endl;			break;
      case B_LAUNCH_FAILED_EXECUTABLE: 			cout << "Application is not executable" << endl;	break;
      case B_LAUNCH_FAILED_APP_IN_TRASH: 		cout << "Application is in trash" << endl;			break;
      case B_LAUNCH_FAILED_NO_PREFERRED_APP: 	cout << "No preferred app" << endl;					break;
      case B_LAUNCH_FAILED_FILES_APP_NOT_FOUND:	cout << "Preferred app AWOL" << endl;				break;
      case B_BAD_MIME_SNIFFER_RULE: 			cout << "MIME type sniffer rule failed" << endl; 	break;
      
      case B_BAD_THREAD_ID:						cout << "Thread id is invalid" << endl;				break;
      case B_BAD_THREAD_STATE:					cout << "Thread in wrong state" << endl;			break;
      case B_NO_MORE_THREADS:					cout << "No more thread ids available" << endl;		break;
      case B_BAD_TEAM_ID:						cout << "Team id is invalid" << endl;				break;
      case B_NO_MORE_TEAMS:						cout << "No more team ids available" << endl;		break;
      case B_BAD_PORT_ID:						cout << "Port id is invalid" << endl;				break;
      case B_NO_MORE_PORTS:						cout << "No more ports are available" << endl;		break;
      case B_BAD_SEM_ID:						cout << "Semaphore id is invalid" << endl;			break;
      case B_NO_MORE_SEMS:						cout << "No more semaphores are available" << endl;	break;
      case B_BAD_IMAGE_ID:						cout << "Image id is invalid" << endl;				break;
      case B_NOT_AN_EXECUTABLE:					cout << "Image is not executable" << endl;			break;
      case B_BAD_ADDRESS:						cout << "Illegal address" << endl;					break;
      case B_MISSING_LIBRARY:					cout << "Library needed by app missing" << endl;	break;
      case B_MISSING_SYMBOL:					cout << "Symbol needed by app missing" << endl;		break;
      case B_DEBUGGER_ALREADY_INSTALLED:		cout << "Debugger already installed" << endl;		break;
      
      case B_MEDIA_SYSTEM_FAILURE:				cout << "General media system collapse" << endl;	break;
      case B_MEDIA_BAD_NODE:					cout << "Invalid or nonexistent node" << endl;		break;
      case B_MEDIA_NODE_BUSY:					cout << "Busy node" << endl;						break;
      case B_MEDIA_BAD_FORMAT:					cout << "Invalid media format" << endl;				break;
      case B_MEDIA_TOO_MANY_NODES:				cout << "No more nodes available" << endl;			break;
      case B_MEDIA_TOO_MANY_BUFFERS:			cout << "No more buffers available" << endl;		break;
      case B_MEDIA_NODE_ALREADY_EXISTS:			cout << "Buffer already exists" << endl;			break;
      case B_MEDIA_CANNOT_SEEK:					cout << "Seek failure" << endl;						break;
      case B_MEDIA_CANNOT_CHANGE_RUN_MODE:		cout << "Run mode fixed" << endl;					break;
      case B_MEDIA_APP_ALREADY_REGISTERED:		cout << "Application already registered" << endl;	break;
      case B_MEDIA_CANNOT_RECLAIM_BUFFERS:		cout << "Cannot reclaim buffer" << endl;			break;
      case B_MEDIA_TIME_SOURCE_STOPPED:			cout << "Time source is busy" << endl;				break;
      case B_MEDIA_BAD_SOURCE:					cout << "Invalid media source" << endl;				break;
      case B_MEDIA_BAD_DESTINATION:				cout << "Invalid media destination" << endl;		break;
      case B_MEDIA_NOT_CONNECTED:				cout << "No connection" << endl;					break;
      case B_MEDIA_BAD_CLIP_FORMAT:				cout << "Invalid clip format" << endl;				break;
      case B_MEDIA_ADDON_FAILED:				cout << "Failed media addon" << endl;				break;
      case B_MEDIA_ADDON_DISABLED:				cout << "Disabled media addon" << endl;				break;
      case B_MEDIA_CHANGE_IN_PROGRESS:			cout << "Conflicting, concurrent change" << endl;	break;
      case B_MEDIA_STALE_CHANGE_COUNT:			cout << "Stale change count" << endl;				break;
      case B_MEDIA_ADDON_RESTRICTED:			cout << "Restricted media addon" << endl;			break;
      case B_MEDIA_NO_HANDLER:					cout << "No handler available" << endl;				break;
      case B_MEDIA_DUPLICATE_FORMAT:			cout << "The specified format exists" << endl;		break;
      case B_MEDIA_REALTIME_DISABLED:			cout << "Realtime allocation disabled" << endl;		break;
      case B_MEDIA_REALTIME_UNAVAILABLE:		cout << "Realtime allocation unavailable" << endl;	break;
      
      case B_MAIL_NO_DAEMON:					cout << "mail_daemon not running" << endl;			break;
      case B_MAIL_UNKNOWN_USER:					cout << "Unknown user" << endl;						break;
      case B_MAIL_WRONG_PASSWORD:				cout << "Password does not match user" << endl;		break;
      case B_MAIL_UNKNOWN_HOST:					cout << "Host unrecognized" << endl;				break;
      case B_MAIL_ACCESS_ERROR:					cout << "Host access error" << endl;				break;
      case B_MAIL_UNKNOWN_FIELD:				cout << "Unknown message field name" << endl;		break;
      case B_MAIL_NO_RECIPIENT:					cout << "Address unknown" << endl;					break;
      case B_MAIL_INVALID_MAIL:					cout << "Invalid mail" << endl;						break;
      
      case B_DEV_INVALID_IOCTL:					cout << "Invalid IOCTL" << endl;					break;
      case B_DEV_NO_MEMORY:						cout << "No memory for device" << endl;				break;
      case B_DEV_BAD_DRIVE_NUM:					cout << "Bad drive number" << endl;					break;
      case B_DEV_NO_MEDIA:						cout << "No media in device" << endl;				break;
      case B_DEV_UNREADABLE:					cout << "Unreadable device" << endl;				break;
      case B_DEV_FORMAT_ERROR:					cout << "Device format error" << endl;				break;
      case B_DEV_TIMEOUT:						cout << "Device timeout" << endl;					break;
      case B_DEV_RECALIBRATE_ERROR:				cout << "Device recalibrate error" << endl;			break;
      case B_DEV_SEEK_ERROR:					cout << "Device seek error" << endl;				break;
      case B_DEV_ID_ERROR:						cout << "Device ID error" << endl;					break;
      case B_DEV_READ_ERROR:					cout << "Device read error" << endl;				break;
      case B_DEV_NOT_READY:						cout << "Device not ready" << endl;					break;
      case B_DEV_MEDIA_CHANGE_REQUESTED:		cout << "Media change requested" << endl;			break;
      case B_DEV_RESOURCE_CONFLICT:				cout << "Device resource conflict" << endl;			break;
      case B_DEV_CONFIGURATION_ERROR:			cout << "Device configuration error" << endl;		break;
      case B_DEV_DISABLED_BY_USER:				cout << "Device disabled by user" << endl;			break;
      case B_DEV_DOOR_OPEN:						cout << "Shut da door baby" << endl;				break;
      default:
        if (MoreErrors)
          MoreErrors(status);
    };
  };  
};