#define __BUILDING_PERCOLINKER_H 1

#include <Messenger.h>
#include "Verbosity.h"
#include "PercoView.h"
#include "PercoLinker.h"
#include "PercolatorConduit.h"
#include "PercolatorCodes.h"

BPercoLinker::BPercoLinker(const char *name, BMessenger *inf_lnk_percolator = NULL)
  :BLooper(name)
{
  if (inf_lnk_percolator)
    lnk_Percolator = new BMessenger(*inf_lnk_percolator);
  else
    lnk_Percolator = new BMessenger("application/x-vnd.Be-PERC", -1);
};

BPercoLinker::~BPercoLinker()
{
  if (lnk_Percolator)
    delete lnk_Percolator;
};

void BPercoLinker::MessageReceived(BMessage *msg_received)
{
  switch (msg_received->what)
  {
    case B_PERCOLATOR_REDIRECT:
      Redirect(msg_received);
    break;
    case B_PERCOLATOR_PING:
      PingReceived(msg_received);
    break;
    case B_PERCOLATOR_RESIZE_NOTIFY:
      FrameResized(msg_received);
    break;
    default:
      BLooper::MessageReceived(msg_received);
  };
};

void BPercoLinker::AssociateConduit(BPercolatorConduit *lnk_conduit)
{
  lnk_Conduit = lnk_conduit;
  rct_StartFrame = lnk_conduit->rct_StartFrame;
};

void BPercoLinker::PingReceived(BMessage *msg_ping)
{
  cout << "Pinged" << endl;
  BMessage *msg_PingReply = new BMessage(B_PERCOLATOR_PING_REPLY);
    lnk_Percolator->SendMessage(msg_PingReply, this);
  delete msg_PingReply;
};

void BPercoLinker::Redirect(BMessage *msg_redirect)
{
  if (lnk_Percolator)
    delete lnk_Percolator;
  lnk_Percolator = new BMessenger();
  msg_redirect->FindMessenger("Be:PERC:redirect",lnk_Percolator);
  lnk_Conduit->SwapConnection(lnk_Percolator);
  
  BMessage *msg_FrameStatus = new BMessage(B_PERCOLATOR_RESIZE_NOTIFY);
    msg_FrameStatus->AddFloat("Be:PERC:width", rct_StartFrame.Width());
    msg_FrameStatus->AddFloat("Be:PERC:height", rct_StartFrame.Height());
    lnk_Percolator->SendMessage(msg_FrameStatus, this);
  delete msg_FrameStatus;
};

void BPercoLinker::FrameResized(BMessage *msg_resize)
{
  cout << "resized" << endl;
  lnk_Percolator->SendMessage(msg_resize);
};