#define __BUILDING_PRECOLATOR_CONDUIT_H

#include "PercolatorConduit.h"
#include <iostream.h>
#include <Messenger.h>
#include <String.h>
#include <Window.h>
#include <View.h>
#include "Verbosity.h"
#include "PercoView.h"
#include "PercoLinker.h"
#include "PercolatorCodes.h"

//constructors and destructors:

BPercolatorConduit::BPercolatorConduit(BRect frame, const char *name, uint32 resizingMode, uint32 flags, uint32 pflags = 0)
{
  flg_Settings = pflags;						 	//set up the flags.
  flg_Status = B_PERCO_START;
  
  rct_StartFrame = frame;							//set up local variables.
  
  if (flg_Settings & B_PERCO_NOTICES)			//report.
    cout << "Starting Conduit" << endl;
  
  lnk_Percolator = NULL;				//set up the messenger
  lnk_Percolator = new BMessenger("application/x-vnd.Be-PERC", -1, &rpt_Token);
  if (flg_Settings & B_PERCO_REPORT)
    VerboseReply(rpt_Token, "Making Messenger", flg_Settings & B_PERCO_INFORMATION);
    
  BString LooperName("p>");
  LooperName << name;  
  thr_Looper = NULL;					//set up the looper
  thr_Looper = new BPercoLinker(LooperName.String(), lnk_Percolator);
  if (flg_Settings & B_PERCO_NOTICES)
    cout << "Starting looper" << endl;
  thr_Looper->Run();					//launch the looper
  thr_Looper->AssociateConduit(this);	//Associate the looper
    
  lnk_This = new BMessenger(NULL, thr_Looper);	//create the "This" messenger.  
  
  BString ViewName("v>");
  ViewName << name;
  if (flg_Settings & B_PERCO_NOTICES)
    cout << "Creating View" << endl;
  vww_Output = new BPercoView(frame, ViewName.String(), resizingMode, flags);
  vww_Output->AssociateConduit(thr_Looper);	//Associate the view.

  str_Name = new BString(name);
};

BPercolatorConduit::BPercolatorConduit(BView *vww_replace, uint32 pflags = 0)
{
  flg_Settings = pflags;						 	//set up the flags.
  flg_Status = B_PERCO_START;

  if (flg_Settings & B_PERCO_NOTICES)			//report.
    cout << "Starting Conduit" << endl;
  
  lnk_Percolator = NULL;				//set up the messenger
  lnk_Percolator = new BMessenger("application/x-vnd.Be-PERC", -1, &rpt_Token);
  if (flg_Settings & B_PERCO_REPORT)
    VerboseReply(rpt_Token, "Making Messenger", flg_Settings & B_PERCO_INFORMATION);

  BString LooperName("p>");
  LooperName << vww_replace->Name();  
  thr_Looper = NULL;					//set up the looper
  thr_Looper = new BPercoLinker(LooperName.String(), lnk_Percolator);
  if (flg_Settings & B_PERCO_NOTICES)
    cout << "Starting looper" << endl;
  thr_Looper->Run();					//launch the looper
  thr_Looper->AssociateConduit(this);	//Associate the conduit.
  
  lnk_This = new BMessenger(NULL, thr_Looper);	//create the "This" messenger.  
  
  BString ViewName("v>");
  ViewName << vww_replace->Name();
  if (vww_replace)						//if we have a view template...
  {
    BMessage *tmp_msg_Archive = new BMessage('tmpa');		//archive it for the info
      rpt_Token = vww_replace->Archive(tmp_msg_Archive);		
      if (flg_Settings & B_PERCO_REPORT)
        VerboseReply(rpt_Token, "Archiving old view", flg_Settings & B_PERCO_INFORMATION);
      BWindow *lnk_win_Parent = vww_replace->Window();		//Identify its old parent.
      vww_Output = (BPercoView *)BPercoView::Instantiate(tmp_msg_Archive);//Create the Percified clone.
      if (lnk_win_Parent)
      {
        lnk_win_Parent->RemoveChild(vww_replace);			//swapping.
        lnk_win_Parent->AddChild(vww_Output);
      };
      delete vww_replace;								//kill the template view
      vww_Output->SetName(ViewName.String());			//rename
      str_Name = new BString(ViewName.String());
      rct_StartFrame = vww_Output->Frame();				//set up the frame variable.
      
      vww_Output->AssociateConduit(thr_Looper);
    delete tmp_msg_Archive;
  }
  else									//or else start from scratch, like before.
    if (flg_Settings & B_PERCO_NOTICES)
    {
      cout << "Old view not found..." << endl;
      flg_Settings |= B_PERCO_NO_VIEW;			//throw the "no view" flag.
    };
};

BPercolatorConduit::~BPercolatorConduit()
{
  if (str_Typecode)
    delete str_Typecode;
  if (str_Name)
    delete str_Name;

  rpt_Token = TerminateConnection();	//close the connection
  if (flg_Settings & B_PERCO_REPORT)
    VerboseReply(rpt_Token, "Closing Connection", flg_Settings & B_PERCO_INFORMATION);
    
  if (lnk_Percolator)					//destroy the messengers
    delete lnk_Percolator;    
  delete lnk_This;
};

//control functions:

status_t BPercolatorConduit::SendFile(entry_ref &refs)
{
  BMessage *msg_FileNotice = new BMessage(B_PERCOLATOR_FILE_REQUEST);
    msg_FileNotice->AddRef("Be:PERC:fileref", &refs);
    lnk_Percolator->SendMessage(msg_FileNotice, *lnk_This);
  delete msg_FileNotice;
  return B_OK;
};

status_t BPercolatorConduit::SendFile(BPath path)
{
  entry_ref ref_converted;
  status_t err_conversion;
  
  BMessage *msg_FileNotice = new BMessage(B_PERCOLATOR_FILE_REQUEST);
    err_conversion = get_ref_for_path(path.Path(), &ref_converted);	//convert path->ref
    msg_FileNotice->AddRef("Be:PERC:fileref", &ref_converted);		//add to messsage
    lnk_Percolator->SendMessage(msg_FileNotice, *lnk_This);			//send it off!
  delete msg_FileNotice;
  
  return err_conversion;
};

void BPercolatorConduit::SetFlags(int32 flags)
{
  flg_Settings = flags;
};

status_t BPercolatorConduit::GetStatus()
{
  //not yet implemented.
  return B_OK;
};

status_t BPercolatorConduit::Initialize(const char *type)
{
  str_Typecode = new BString(type);
  
  //start up the connection...
  BMessage *msg_StartConduit = new BMessage(B_PERCOLATOR_START_CONDUIT);
    msg_StartConduit->AddString("Be:PERC:name",*str_Name);
    msg_StartConduit->AddString("Be:PERC:mimetype",*str_Typecode);
    msg_StartConduit->AddMessenger("Be:PERC:linkback",*lnk_This);
    rpt_Token = lnk_Percolator->SendMessage(msg_StartConduit, *lnk_This);
  delete msg_StartConduit;

  if (flg_Settings & B_PERCO_REPORT)
    VerboseReply(rpt_Token, "Starting Connection", flg_Settings & B_PERCO_INFORMATION);
  
  return rpt_Token;
};

status_t BPercolatorConduit::ResetType(const char *mimetype)
{
  str_Typecode = new BString(mimetype);
  
  //start up the connection...
  BMessage *msg_StartConduit = new BMessage(B_PERCOLATOR_TYPECHANGE_NOTIFY);
    msg_StartConduit->AddString("Be:PERC:mimetype",*str_Typecode);
    rpt_Token = lnk_Percolator->SendMessage(msg_StartConduit, *lnk_This);
  delete msg_StartConduit;

  if (flg_Settings & B_PERCO_REPORT)
    VerboseReply(rpt_Token, "Starting Connection", flg_Settings & B_PERCO_INFORMATION);
  
  return rpt_Token;
};

status_t BPercolatorConduit::TerminateConnection()
{
  BMessage *msg_TerminateConnection = new BMessage(B_PERCOLATOR_TERMINATE_REQUEST);
    rpt_Token = lnk_Percolator->SendMessage(msg_TerminateConnection, *lnk_This);
  delete msg_TerminateConnection;
  
  return rpt_Token;
};

status_t BPercolatorConduit::SuspendConnection()
{
  BMessage *msg_SuspendConnection = new BMessage(B_PERCOLATOR_SUSPEND_REQUEST);
    rpt_Token = lnk_Percolator->SendMessage(msg_SuspendConnection, *lnk_This);
  delete msg_SuspendConnection;
  
  return B_OK;
};

status_t BPercolatorConduit::RestartConnection()
{
  BMessage *msg_RestartConnection = new BMessage(B_PERCOLATOR_REACTIVATE_REQUEST);
    rpt_Token = lnk_Percolator->SendMessage(msg_RestartConnection, *lnk_This);
  delete msg_RestartConnection;
  
  return B_OK;
};

void BPercolatorConduit::SwapConnection(BMessenger *lnk_percolator)
{
  *lnk_Percolator = *lnk_percolator;  
};

BPercoView *BPercolatorConduit::View()
{
  return vww_Output;
};